@@ -149,8 +149,10 @@ from six.moves.urllib.parse import parse_qs
 from swift.common.constraints import valid_api_version
 from swift.common.middleware.listing_formats import \
     MAX_CONTAINER_LISTING_CONTENT_LENGTH
+from swift.common.request_helpers import append_log_info
 from swift.common.wsgi import PipelineWrapper, loadcontext, WSGIContext
 
+from swift.common.middleware import app_property
 from swift.common.middleware.s3api.exception import NotS3Request, \
     InvalidSubresource
 from swift.common.middleware.s3api.s3request import get_request_class
@@ -168,6 +170,12 @@ class ListingEtagMiddleware(object):
     def __init__(self, app):
         self.app = app
 
+    # Pass these along so get_container_info will have the configured
+    # odds to skip cache
+    _pipeline_final_app = app_property('_pipeline_final_app')
+    _pipeline_request_logging_app = app_property(
+        '_pipeline_request_logging_app')
+
     def __call__(self, env, start_response):
         # a lot of this is cribbed from listing_formats / swob.Request
         if env['REQUEST_METHOD'] != 'GET':
@@ -290,7 +298,7 @@ class S3ApiMiddleware(object):
             wsgi_conf.get('ratelimit_as_client_error', False))
 
         self.logger = get_logger(
-            wsgi_conf, log_route=wsgi_conf.get('log_name', 's3api'))
+            wsgi_conf, log_route='s3api', statsd_tail_prefix='s3api')
         self.check_pipeline(wsgi_conf)
 
     def is_s3_cors_preflight(self, env):
@@ -345,6 +353,8 @@ class S3ApiMiddleware(object):
         except InvalidSubresource as e:
             self.logger.debug(e.cause)
         except ErrorResponse as err_resp:
+            self.logger.increment(err_resp.metric_name)
+            append_log_info(env, 's3:err:%s' % err_resp.summary)
             if isinstance(err_resp, InternalError):
                 self.logger.exception(err_resp)
             resp = err_resp
@@ -358,6 +368,7 @@ class S3ApiMiddleware(object):
 
         if 's3api.backend_path' in env and 'swift.backend_path' not in env:
             env['swift.backend_path'] = env['s3api.backend_path']
+
         return resp(env, start_response)
 
     def handle_request(self, req):
@@ -382,6 +393,9 @@ class S3ApiMiddleware(object):
             raise MethodNotAllowed(req.method,
                                    req.controller.resource_type())
 
+        if req.policy_index is not None:
+            res.headers.setdefault('X-Backend-Storage-Policy-Index',
+                                   req.policy_index)
         return res
 
     def check_pipeline(self, wsgi_conf):
