@@ -48,6 +48,7 @@ import six
 import six.moves.cPickle as pickle
 from six.moves import range
 from six.moves.http_client import HTTPException
+from six.moves import configparser
 
 from swift.common import storage_policy, swob, utils, exceptions
 from swift.common.memcached import MemcacheConnectionError
@@ -1049,8 +1050,11 @@ def mocked_http_conn(*args, **kwargs):
         if left_over_status:
             raise AssertionError('left over status %r' % left_over_status)
         if fake_conn.unexpected_requests:
-            raise AssertionError('unexpected requests:\n%s' % '\n  '.join(
-                '%r' % (req,) for req in fake_conn.unexpected_requests))
+            raise AssertionError(
+                '%d unexpected requests:\n%s' %
+                (len(fake_conn.unexpected_requests),
+                 '\n  '.join('%r' % (req,)
+                             for req in fake_conn.unexpected_requests)))
 
 
 def make_timestamp_iter(offset=0):
@@ -1448,7 +1452,7 @@ class ConfigAssertMixin(object):
 
     def assertDuplicateOptionError(self, app_config, option_name):
         with self.assertRaises(
-                utils.configparser.DuplicateOptionError) as ctx:
+                configparser.DuplicateOptionError) as ctx:
             app_config()
         msg = str(ctx.exception)
         self.assertIn(option_name, msg)
