@@ -25,10 +25,12 @@ from textwrap import dedent
 import six
 from six.moves import range, zip_longest
 from six.moves.urllib.parse import quote, parse_qsl
-from swift.common import exceptions, internal_client, request_helpers, swob
+from swift.common import exceptions, internal_client, request_helpers, swob, \
+    utils
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.storage_policy import StoragePolicy
 from swift.common.middleware.proxy_logging import ProxyLoggingMiddleware
+from swift.common.middleware.gatekeeper import GatekeeperMiddleware
 
 from test.debug_logger import debug_logger
 from test.unit import with_tempdir, write_fake_ring, patch_policies
@@ -237,7 +239,6 @@ class TestInternalClient(unittest.TestCase):
 
         [app:proxy-server]
         use = egg:swift#proxy
-        auto_create_account_prefix = -
 
         [filter:cache]
         use = egg:swift#memcache
@@ -259,11 +260,6 @@ class TestInternalClient(unittest.TestCase):
             with mock.patch('swift.proxy.server.get_logger',
                             lambda *a, **kw: logger):
                 client = internal_client.InternalClient(conf_path, 'test', 1)
-            self.assertEqual(logger.get_lines_for_level('warning'), [
-                'Option auto_create_account_prefix is deprecated. '
-                'Configure auto_create_account_prefix under the '
-                'swift-constraints section of swift.conf. This option will '
-                'be ignored in a future release.'])
             self.assertEqual(client.account_ring,
                              client.app.app.app.account_ring)
             self.assertEqual(client.account_ring.serialized_path,
@@ -277,7 +273,6 @@ class TestInternalClient(unittest.TestCase):
                              object_ring)
             self.assertEqual(object_ring.serialized_path,
                              object_ring_path)
-            self.assertEqual(client.auto_create_account_prefix, '-')
 
     @mock.patch('swift.common.utils.HASH_PATH_SUFFIX', new=b'endcap')
     @with_tempdir
@@ -301,7 +296,6 @@ class TestInternalClient(unittest.TestCase):
 
         [app:proxy-server]
         use = egg:swift#proxy
-        auto_create_account_prefix = -
 
         [filter:cache]
         use = egg:swift#memcache
@@ -330,7 +324,6 @@ class TestInternalClient(unittest.TestCase):
 
         [app:proxy-server]
         use = egg:swift#proxy
-        auto_create_account_prefix = -
 
         [filter:cache]
         use = egg:swift#memcache
@@ -359,7 +352,6 @@ class TestInternalClient(unittest.TestCase):
 
         [app:proxy-server]
         use = egg:swift#proxy
-        auto_create_account_prefix = -
 
         [filter:cache]
         use = egg:swift#memcache
@@ -383,14 +375,29 @@ class TestInternalClient(unittest.TestCase):
         user_agent = 'some_user_agent'
         request_tries = 123
 
-        with mock.patch.object(
-                internal_client, 'loadapp', return_value=app) as mock_loadapp,\
+        with mock.patch.object(internal_client, 'loadapp',
+                               return_value=app) as mock_loadapp, \
                 self.assertRaises(ValueError):
             # First try with a bad arg
             internal_client.InternalClient(
                 conf_path, user_agent, request_tries=0)
         mock_loadapp.assert_not_called()
 
+        # if we load it with the gatekeeper middleware then we also get a
+        # value error
+        gate_keeper_app = GatekeeperMiddleware(app, {})
+        gate_keeper_app._pipeline_final_app = app
+        gate_keeper_app._pipeline = [gate_keeper_app, app]
+        with mock.patch.object(
+                internal_client, 'loadapp', return_value=gate_keeper_app) \
+                as mock_loadapp, self.assertRaises(ValueError) as err:
+            internal_client.InternalClient(
+                conf_path, user_agent, request_tries)
+        self.assertEqual(
+            str(err.exception),
+            ('Gatekeeper middleware is not allowed in the InternalClient '
+             'proxy pipeline'))
+
         with mock.patch.object(
                 internal_client, 'loadapp', return_value=app) as mock_loadapp:
             client = internal_client.InternalClient(
@@ -420,6 +427,72 @@ class TestInternalClient(unittest.TestCase):
         self.assertEqual(request_tries, client.request_tries)
         self.assertTrue(client.use_replication_network)
 
+    def test_init_allow_modify_pipeline(self):
+        conf_path = 'some_path'
+        app = FakeSwift()
+        user_agent = 'some_user_agent'
+
+        with mock.patch.object(internal_client, 'loadapp',
+                               return_value=app) as mock_loadapp, \
+                self.assertRaises(ValueError) as cm:
+            internal_client.InternalClient(
+                conf_path, user_agent, 1, allow_modify_pipeline=True)
+        mock_loadapp.assert_not_called()
+        self.assertIn("'allow_modify_pipeline' is no longer supported",
+                      str(cm.exception))
+
+        with mock.patch.object(
+                internal_client, 'loadapp', return_value=app) as mock_loadapp:
+            internal_client.InternalClient(
+                conf_path, user_agent, 1, allow_modify_pipeline=False)
+        mock_loadapp.assert_called_once_with(
+            conf_path, allow_modify_pipeline=False, global_conf=None)
+
+    def test_gatekeeper_not_loaded(self):
+        app = FakeSwift()
+        pipeline = [app]
+
+        class RandomMiddleware(object):
+            def __init__(self, app):
+                self.app = app
+                self._pipeline_final_app = app
+                self._pipeline = pipeline
+                self._pipeline.insert(0, self)
+
+        # if there is no Gatekeeper middleware then it's false
+        # just the final app
+        self.assertFalse(
+            internal_client.InternalClient.check_gatekeeper_not_loaded(app))
+
+        # now with a bunch of middlewares
+        app_no_gatekeeper = app
+        for i in range(5):
+            app_no_gatekeeper = RandomMiddleware(app_no_gatekeeper)
+            self.assertFalse(
+                internal_client.InternalClient.check_gatekeeper_not_loaded(
+                    app_no_gatekeeper))
+
+        # But if we put the gatekeeper on the end, it will be found
+        app_with_gatekeeper = GatekeeperMiddleware(app_no_gatekeeper, {})
+        pipeline.insert(0, app_with_gatekeeper)
+        app_with_gatekeeper._pipeline = pipeline
+        with self.assertRaises(ValueError) as err:
+            internal_client.InternalClient.check_gatekeeper_not_loaded(
+                app_with_gatekeeper)
+        self.assertEqual(str(err.exception),
+                         ('Gatekeeper middleware is not allowed in the '
+                          'InternalClient proxy pipeline'))
+
+        # even if we bury deep into the pipeline
+        for i in range(5):
+            app_with_gatekeeper = RandomMiddleware(app_with_gatekeeper)
+            with self.assertRaises(ValueError) as err:
+                internal_client.InternalClient.check_gatekeeper_not_loaded(
+                    app_with_gatekeeper)
+            self.assertEqual(str(err.exception),
+                             ('Gatekeeper middleware is not allowed in the '
+                              'InternalClient proxy pipeline'))
+
     def test_make_request_sets_user_agent(self):
         class FakeApp(FakeSwift):
             def __init__(self, test):
@@ -439,6 +512,38 @@ class TestInternalClient(unittest.TestCase):
             app=FakeApp(self))
         client.make_request('GET', '/', {}, (200,))
 
+    def test_make_request_clears_txn_id_after_calling_app(self):
+        class InternalClient(internal_client.InternalClient):
+            def __init__(self, test, logger):
+                def fake_app(env, start_response):
+                    self.app.logger.txn_id = 'foo'
+                    self.app.logger.debug('Inside of request')
+                    start_response('200 Ok', [('Content-Length', '0')])
+                    return []
+
+                self.test = test
+                self.user_agent = 'some_agent'
+                self.app = fake_app
+                self.app.logger = logger
+                self.request_tries = 1
+                self.use_replication_network = False
+
+        fake_logger = debug_logger()
+        logger = utils.LogAdapter(fake_logger, 'test-server')
+        # Make sure there's no transaction ID set -- other tests may have
+        # polluted the logger
+        logger.txn_id = None
+        logger.debug('Before request')
+        client = InternalClient(self, logger)
+        client.make_request('GET', '/', {}, (200,))
+        logger.debug('After request')
+        self.assertEqual([(args[0], kwargs['extra'].get('txn_id'))
+                          for args, kwargs in fake_logger.log_dict['debug']], [
+            ('Before request', None),
+            ('Inside of request', 'foo'),
+            ('After request', None),
+        ])
+
     def test_make_request_defaults_replication_network_header(self):
         class FakeApp(FakeSwift):
             def __init__(self, test):
@@ -670,7 +775,8 @@ class TestInternalClient(unittest.TestCase):
             client = internal_client.InternalClient(
                 None, "some_agent", 3, use_replication_network=False,
                 app=fake_app)
-            with self.assertRaises(internal_client.UnexpectedResponse) as ctx,\
+            expected_error = internal_client.UnexpectedResponse
+            with self.assertRaises(expected_error) as ctx, \
                     mock.patch('swift.common.internal_client.sleep'):
                 # This is obvious strange tests to expect only 400 Bad Request
                 # but this test intended to avoid extra body drain if it's
@@ -1487,9 +1593,8 @@ class TestInternalClient(unittest.TestCase):
         self.assertEqual(app.call_count, 1)
         req_headers.update({
             'host': 'localhost:80',  # from swob.Request.blank
+            'user-agent': 'test',  # from IC
             'x-backend-allow-reserved-names': 'true',  # also from IC
-            'x-backend-storage-policy-index': '2',  # from proxy-server app
-            'user-agent': 'test',
         })
         self.assertEqual(app.calls_with_headers, [(
             'GET', path_info + '?symlink=get', HeaderKeyDict(req_headers))])
@@ -1931,7 +2036,7 @@ class TestSimpleClient(unittest.TestCase):
                 self.assertEqual(1, len(args))
                 self.assertEqual(1, len(kwargs))
                 self.assertEqual(0.1, kwargs['timeout'])
-                self.assertTrue(isinstance(args[0], urllib2.Request))
+                self.assertIsInstance(args[0], urllib2.Request)
                 self.assertEqual(proxy_host, args[0].host)
                 if six.PY2:
                     self.assertEqual(scheme, args[0].type)
@@ -1956,7 +2061,7 @@ class TestSimpleClient(unittest.TestCase):
                 self.assertEqual(1, len(args))
                 self.assertEqual(1, len(kwargs))
                 self.assertEqual(0.1, kwargs['timeout'])
-                self.assertTrue(isinstance(args[0], urllib2.Request))
+                self.assertIsInstance(args[0], urllib2.Request)
                 self.assertEqual(proxy_host, args[0].host)
                 if six.PY2:
                     self.assertEqual(scheme, args[0].type)
